@@ -2,7 +2,7 @@
  * File: props.c
  * Author: Justin Zaun
  * Project: GGZ GTK Client
- * $Id: props.c,v 1.1 2006/10/14 10:27:32 andreasr Exp $
+ * $Id: props.c 5818 2007-01-05 17:58:19Z andreasr $
  *
  * This is the main program body for the GGZ client
  *
@@ -1425,7 +1425,7 @@ GtkWidget* create_props_dlg(void)
                             (GtkDestroyNotify) gtk_widget_unref);
   gtk_box_pack_start (GTK_BOX (hbox15), info_country, TRUE, TRUE, 5);
 
-  label21 = gtk_label_new (_("Comments, Hobies, Etc."));
+  label21 = gtk_label_new (_("Comments, Hobbies, Etc."));
   gtk_widget_ref (label21);
   g_object_set_data_full(G_OBJECT (dlg_props), "label21", label21,
                             (GtkDestroyNotify) gtk_widget_unref);
