@@ -10,7 +10,9 @@ GLADE_DIR       = os.path.join(APP_DATA_DIR, 'glchess')
 BASE_DIR        = os.path.join(APP_DATA_DIR, 'glchess')
 LOCALEDIR 	= os.path.join(APP_DATA_DIR, 'locale')
 DATA_DIR        = os.path.expanduser('~/.gnome2/glchess/')
+LOG_DIR         = os.path.join(DATA_DIR, 'logs')
 CONFIG_FILE     = os.path.join(DATA_DIR, 'config.xml')
+AUTOSAVE_FILE   = os.path.join(DATA_DIR, 'autosave.pgn')
 LOCAL_AI_CONFIG = os.path.join(DATA_DIR, 'ai.xml')
 
 DOMAIN = 'gnome-games'
@@ -22,7 +24,7 @@ import gtk.glade
 gtk.glade.bindtextdomain (DOMAIN, LOCALEDIR)
 gtk.glade.textdomain (DOMAIN)
 
-VERSION = "2.19.1"
+VERSION = "2.19.2"
 APPNAME = _("glChess")
 
 COPYRIGHT     = _('Copyright 2005-2007 Robert Ancell (and contributors)')
